@@ -27,16 +27,18 @@ class Executor:
         pass
 
     def handle_request_run_application(self, request):
+        log_prefix = "handle_request_run_application"
         # request example
         # {
         #     "action": "run-application",
         #     "id": 123,
-        #     "args": ["foo", "bar"]
+        #     "args": ["foo", "bar"],
+        #     "input": "ls -al\nclear\n"
         # }
         try:
             validate_model('run_application_input', request)
-        except ValidationError:
-            logger.warn("handle_request_run_application: validation error, schema: run_application_input")
+        except ValidationError as e:
+            logger.exception(f"{log_prefix}: validation error, schema: run_application_input, error: {str(e)}")
             return {
                 "status": SERVER_STATUS.INVALID_REQUEST_SCHEMA.value,
                 "message": str(e)
@@ -44,6 +46,7 @@ class Executor:
 
         application_id = request["id"]
         args = request.get('args', [])
+        input = request.get('input')
         run = None
         launched = False
         try:
@@ -51,30 +54,33 @@ class Executor:
                 run = self.dbagent_client.create_run(
                     application_id = application_id,
                     node_id = self.node_id,
-                    args = args
+                    args = args,
+                    input = input,
                 )
             except DBAgentException as e:
-                logger.warn("handle_request_run_application: unable to create run")
-                return {"status": e.respp["status"]}
+                logger.exception(f"{log_prefix}: failed to create run, error: {str(e)}")
+                return {"status": e.resp["status"]}
+            log_prefix = f"{log_prefix}(Run({run['id']})"
+            logger.info(f"{log_prefix}: run created!")
 
             run_dir = os.path.join(self.runs_dir, str(run['id']))
             try:
                 os.makedirs(run_dir)
+                logger.info(f"{log_prefix}: run home directory {run_dir} created!")
             except OSError as e:
-                logger.error(f"handle_request_run_application: unable to create run directory {run_dir}, error: {str(e)}")
+                logger.exception(f"{log_prefix}: failed to create run home directory {run_dir}, error: {str(e)}")
                 return {
                     "status": SERVER_STATUS.INTERNAL_ERROR.value,
                 }
 
             try:
-                r = subprocess.Popen(
-                    f"pwrapper -r {run['id']} --data-dir {self.base_data_dir} --cfg-dir {self.cfg_dir}",
-                    shell = True,
-                )
+                cmd = f"pwrapper -r {run['id']} --data-dir {self.base_data_dir} --cfg-dir {self.cfg_dir}"
+                logger.info(f"{log_prefix}: launch wrapper via command: \"{cmd}\"")
+                r = subprocess.Popen(cmd, shell = True)
                 launched = True
-                logger.info(f"handle_request_run_application: wrapper launched for Run(id={run['id']}), pid={r.pid}")
+                logger.info(f"{log_prefix}: wrapper launched, pid={r.pid}")
             except OSError as e:
-                logger.exception(f"handle_request_run_application: process launch failed for Run(id={run['id']}), error: {str(e)}")
+                logger.exception(f"{log_prefix}: failed to launch wrapper, error: {str(e)}")
                 return {
                     "status": SERVER_STATUS.LAUNCH_PROCESS_FAILED.value,
                     "message": str(e)
@@ -87,18 +93,18 @@ class Executor:
             }
         finally:
             if not launched:
-                logger.info(f"handle_request_run_application: cleanup on launch failure!")
+                log_prefix = f"{log_prefix}(cleanup)"
                 if run is not None:
                     try:
                         self.dbagent_client.delete_run(run['id'])
-                        logger.warn(f"handle_request_run_application: unset_active_run({task['id']}) succeeded")
+                        logger.warn(f"{log_prefix}: run({run['id']}) is deleted!")
                     except DBAgentException as e:
-                        # dounle error, absorb it
-                        logger.warn(f"handle_request_run_application: unset_active_run({task['id']}) failed")
+                        logger.exception(f"{log_prefix}: failed to delete run({run['id']}), error: {str(e)}")
 
 
     # execute an application using task
     def handle_request_run_task(self, request):
+        log_prefix = "handle_request_run_task"
         # request example
         # {
         #     "action": "run-task",
@@ -107,7 +113,7 @@ class Executor:
         try:
             validate_model('run_task_input', request)
         except ValidationError as e:
-            logger.warn("handle_request_run_task: validation error, schema: run_task_input")
+            logger.exception(f"{log_prefix}: validation error, schema: run_task_input, error: {str(e)}")
             return {
                 "status": SERVER_STATUS.INVALID_REQUEST_SCHEMA.value,
                 "message": str(e)
@@ -117,8 +123,8 @@ class Executor:
         try:
             task = self.dbagent_client.get_task(task_id)
         except DBAgentException as e:
-            logger.warn("handle_request_run_task: unable to get task, task_id={task_id}")
-            return {"status": e.respp["status"]}
+            logger.exception(f"{log_prefix}: failed to get task({task_id}), error: {str(e)}")
+            return {"status": e.resp['status']}
 
         application = task["application"]
         run = None
@@ -130,38 +136,45 @@ class Executor:
                     application_id = application['id'],
                     node_id = self.node_id,
                     args = task['args'],
-                    task_id = task['id']
+                    input= task['input'],
+                    task_id = task['id'],
                 )
             except DBAgentException as e:
-                logger.warn("handle_request_run_task: create_run failed")
-                return {"status": e.respp["status"]}
+                logger.exception(f"{log_prefix}: failed to create run, error: {str(e)}")
+                return {"status": e.resp['status']}
+            log_prefix = f"{log_prefix}(Run({run['id']})"
+            logger.info(f"{log_prefix}: run created!")
 
             if task['is_singleton']:
                 try:
                     self.dbagent_client.set_active_run(task['id'], run_id = run['id'])
                     active_run_set = True
                 except DBAgentException as e:
-                    logger.warn("handle_request_run_task: set_active_run failed")
-                    return {"status": e.respp["status"]}
+                    logger.exception(f"{log_prefix}: failed to acquire task lock, error: {str(e)}")
+                    return {"status": e.resp['status']}
+                logger.info(f"{log_prefix}: task({task['id']}) lock acquired")
 
             run_dir = os.path.join(self.runs_dir, str(run['id']))
             try:
                 os.makedirs(run_dir)
+                logger.info(f"{log_prefix}: run home directory {run_dir} created!")
             except OSError as e:
-                logger.error(f"handle_request_run_task: unable to create run directory {run_dir}, error: {str(e)}")
+                logger.exception(f"{log_prefix}: failed to create run home directory {run_dir}, error: {str(e)}")
                 return {
                     "status": SERVER_STATUS.INTERNAL_ERROR.value,
                 }
 
             try:
-                r = subprocess.Popen(
-                    f"pwrapper -r {run['id']} --data-dir {self.base_data_dir} --cfg-dir {self.cfg_dir}",
-                    shell = True,
-                )
+                if task['is_singleton']:
+                    cmd = f"pwrapper -r {run['id']} -t {task['id']} --is-singleton --data-dir {self.base_data_dir} --cfg-dir {self.cfg_dir}"
+                else:
+                    cmd = f"pwrapper -r {run['id']} -t {task['id']} --data-dir {self.base_data_dir} --cfg-dir {self.cfg_dir}"
+                logger.info(f"{log_prefix}: launch wrapper via command: \"{cmd}\"")
+                r = subprocess.Popen(cmd, shell = True)
                 launched = True
-                logger.info(f"handle_request_run_task: wrapper launched for Run(id={run['id']}), pid={r.pid}")
+                logger.info(f"{log_prefix}: wrapper launched, pid={r.pid}")
             except OSError as e:
-                logger.exception(f"handle_request_run_task: process launch failed for Run(id={run['id']}), error: {str(e)}")
+                logger.exception(f"{log_prefix}: failed to launch wrapper, error: {str(e)}")
                 return {
                     "status": SERVER_STATUS.LAUNCH_PROCESS_FAILED.value,
                     "message": str(e)
@@ -174,24 +187,23 @@ class Executor:
             }
         finally:
             if not launched:
-                logger.info(f"handle_request_run_task: cleanup on launch failure!")
+                log_prefix = f"{log_prefix}(cleanup)"
                 if active_run_set:
                     try:
                         self.dbagent_client.unset_active_run(task['id'])
-                        logger.warn(f"handle_request_run_task: unset_active_run({task['id']}) succeeded")
+                        logger.warn(f"{log_prefix}: task lock released")
                     except DBAgentException as e:
-                        # dounle error, absorb it
-                        logger.warn(f"handle_request_run_task: unset_active_run({task['id']}) failed")
+                        logger.exception(f"{log_prefix}: failed to relase task lock, error: {str(e)}")
                 if run is not None:
                     try:
                         self.dbagent_client.delete_run(run['id'])
-                        logger.warn(f"handle_request_run_task: delete_run({run['id']}) succeeded")
+                        logger.warn(f"{log_prefix}: run({run['id']}) is deleted!")
                     except DBAgentException as e:
-                        # dounle error, absorb it
-                        logger.warn(f"handle_request_run_task: delete_run({run['id']}) failed")
+                        logger.exception(f"{log_prefix}: failed to delete run({run['id']}), error: {str(e)}")
 
 
     def handle_request_stop_run(self, request):
+        log_prefix = "handle_request_stop_run"
         # request example
         # {
         #     "action": "stop-run",
@@ -200,7 +212,7 @@ class Executor:
         try:
             validate_model('stop_run_input', request)
         except ValidationError as e:
-            logger.warn("handle_request_stop_run: validation error, schema: stop_run_input")
+            logger.exception(f"{log_prefix}: validation error, schema: stop_run_input, error: {str(e)}")
             return {
                 "status": SERVER_STATUS.INVALID_REQUEST_SCHEMA.value,
                 "message": str(e)
@@ -210,25 +222,27 @@ class Executor:
         try:
             run = self.dbagent_client.get_run(id)
         except DBAgentException as e:
-            logger.warn("handle_request_stop_run: unable to get run, run_id={id}")
-            return {"status": e.respp["status"]}
+            logger.exception(f"{log_prefix}: failed to get run, run_id={id}, error: {str(e)}")
+            return {"status": e.resp['status']}
+        log_prefix = f"{log_prefix}(Run({run['id']})"
 
         if run["is_finished"]:
-            logger.error(f"handle_request_stop_run: Run(id={id}) is already stopped")
+            logger.warn(f"{log_prefix}: run is already stopped")
             return {
                 "status": SERVER_STATUS.RUN_ALREADY_STOPPED.value,
             }
 
         if run["pid"] is None:
-            logger.error(f"handle_request_stop_run: Run(id={id}) is missing pid")
+            logger.error(f"{log_prefix}: missing pid")
             return {
                 "status": SERVER_STATUS.INTERNAL_ERROR.value,
             }
 
         try:
+            logger.info(f"{log_prefix}: send SIGTERM to pid({run['pid']})")
             os.kill(run["pid"], signal.SIGTERM)
         except OSError as e:
-            logger.exception(f"handle_request_stop_run: failed to send SIGTERM to process for Run(id={run['id']}, pid={run['pid']})")
+            logger.exception(f"{log_prefix}: failed to send SIGTERM to process(pid={run['pid']}), error: {str(e)}")
             return {
                 "status": SERVER_STATUS.STOP_PROCESS_FAILED.value,
                 "message": str(e)
@@ -242,7 +256,12 @@ class Executor:
     def handle_request(self, request):
         logger.info(f"request: {request}")
         action = request.get('action')
-        if action == "run-application":
+        if action == "ping":
+            response = {
+                "status": SERVER_STATUS.OK.value,
+                "payload": "pong"
+            }
+        elif action == "run-application":
             response = self.handle_request_run_application(request)
         elif action == "run-task":
             response = self.handle_request_run_task(request)
